/*******************************************************************************
********************************************************************************
**                                                                            **
**  Turning the power on starts a roughly 90 minute countdown clock.  During  **
**  the first 10 seconds, the user can press the button to  shorten the timer **
**  delay (in approximately 10-minute intervals), or press and hold to test   **
**  all the off codes in the TV-B-Gone library.                               **
**  If the TV goes off and then back on, it must be responding to two codes.  **
**  In this case, the user can press the button to stop the test after the    **
**  TV shuts off.  This will stop the test, and the device will stop at the   **
**  same point after the timer has run down.                                  **
**                                                                            **
**  After the 10-second window closes, the device will count down to zero,    **
**  and then run through all of the TV "off" codes.                           **
**                                                                            **
********************************************************************************
*******************************************************************************/

#include "main.h"
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/wdt.h>

/*
TV-B-Gone for Arduino version 0.001
Ported to Arduino by Ken Shirriff, Dec 3, 2009
http://arcfn.com

The original code is:
TV-B-Gone Firmware version 1.2
 for use with ATtiny85v and v1.2 hardware
 (c) Mitch Altman + Limor Fried 2009
 Last edits, August 16 2009
 
 With some code from:
 Kevin Timmerman & Damien Good 7-Dec-07
 
 Distributed under Creative Commons 2.5 -- Attib & Share Alike
 
 This is the 'universal' code designed for v1.2 - it will select EU or NA
 depending on a pulldown resistor on pin B1 !
 */

void xmitCodeElement(uint16_t ontime, uint16_t offtime, uint8_t PWM_code );
void flashslowLEDx( uint8_t num_blinks );
void quickflashLEDx( uint8_t x );
void tvbgone_sleep( void );
void delay_ten_us(uint16_t us);
void quickflashLED( void );
uint8_t read_bits(uint8_t count);

#define putstring_nl(s) Serial.println(s)
#define putstring(s) Serial.print(s)
#define putnum_ud(n) Serial.print(n, DEC)
#define putnum_uh(n) Serial.print(n, HEX)

/*
This project transmits a bunch of TV POWER codes, one right after the other, 
 with a pause in between each.  (To have a visible indication that it is 
 transmitting, it also pulses a visible LED once each time a POWER code is 
 transmitted.)  That is all TV-B-Gone does.  The tricky part of TV-B-Gone 
 was collecting all of the POWER codes, and getting rid of the duplicates and 
 near-duplicates (because if there is a duplicate, then one POWER code will 
 turn a TV off, and the duplicate will turn it on again (which we certainly 
 do not want).  I have compiled the most popular codes with the 
 duplicates eliminated, both for North America (which is the same as Asia, as 
 far as POWER codes are concerned -- even though much of Asia USES PAL video) 
 and for Europe (which works for Australia, New Zealand, the Middle East, and 
 other parts of the world that use PAL video).
 
 Before creating a TV-B-Gone Kit, I originally started this project by hacking 
 the MiniPOV kit.  This presents a limitation, based on the size of
 the Atmel ATtiny2313 internal flash memory, which is 2KB.  With 2KB we can only 
 fit about 7 POWER codes into the firmware's database of POWER codes.  However,
 the more codes the better! Which is why we chose the ATtiny85 for the 
 TV-B-Gone Kit.
 
 This version of the firmware has the most popular 100+ POWER codes for 
 North America and 100+ POWER codes for Europe. You can select which region 
 to use by soldering a 10K pulldown resistor.
 */


/*
This project is a good example of how to use the AVR chip timers.
 */


/*
The hardware for this project uses an Arduino:
 Connect an IR LED to pin 3.
 Connect a visible LED to pin 13 (or use builtin LED in some Arduinos).
 Connect a pushbutton between pin 1 and ground.
 Pin 0 is floating for North America, or wired to ground for Europe.
 */
const int Button = 2;                  // the number of the pushbutton pin
const int SegA = 10;                   //   -
const int SegB = 12;                   //    |
const int SegC = 5;                    //    |
const int SegD = 6;                    //    + The 7 segments of the LED's numeric display
const int SegE = 4;                    //    |
const int SegF = 9;                    //    |
const int SegG = 8;                    //   -
const int SegDP = 13;                  //  the display's decimal point

const long InactivityTimeOut = 10000;  //  a wait period (in milliseconds) during which the user can shorten the timer delay or start a test, after this period ends the countdown begins 
const long PressOnlyDelay = 50;        //  a brief delay (in milliseconds) to debounce the "press" option - lengthen if "press" option executes multiple times per physical press; shorten if button seems unresponsive
const long PressAndHoldDelay = 500;    //  an extended delay (in milliseconds) to signal the time the button must be held for the "press-and-hold" option to execute - lengthen if press-and-hold option occurs when only a press was intended; shorten if hold-time seems too long
//                                            NOTE: The PressAndHoldDelay must be longer than the PressOnlyDelay.
long FirstDownAt;                      //  a variable to hold the time the button was first found to be pressed, used both for de-bouncing "press" and timing "press-and-hold"
long WillBeInaciveAt;                  //  a variable to indicate the time when the countdown is to begin

boolean AlreadyDown = false;           // a variable used to see if this is not the first time the physical button was found to be pressed, used to keep the debounce clock from re-starting when the same physical pressing is noted multiple times
boolean ButtonWasPressed = false;      // a variable to tell if the button was pressed throughout the debounce interval
boolean ButtonWasHeld = false;         // a variable to tell if the button was pressed and held throughout the press-and-hold interval
boolean Testing = false;               // a variable indicating whether the codes are being sent as a test (or if the timer has elapsed and the TV is being shut down for the night
boolean DoneForTheNight = false;       // a variable indicating that the codes have been sent and the device will no longer be needed

int SleepCyclesLeft = 675;             // a variable to keep track of the number of 8-second power-down cycles remaining until the off codes are to be sent (675 times 8 seconds equates to a 90-minute timer, though the actual time will be longer because a few lines of codes are executed every 8 seconds)
long ShutOffTime;                      // a variable indicating the time when the off codes are to be sent
boolean CountingDown = false;          // a variable indicating that the 10-second programing period has ellapsed and the count-down period has begun 
int NumberOfCodesToUse;                // a variable to keep track of how many codes to send before stopping - the default is all codes, but if the button is pressed during the test, this variable limits the number of codes sent thereafter (to avoid turning a TV off and then back on)

volatile int f_wdt=1;

int LCDdigit[11][7]={                  // an array indicating which segments are illuminated for the display of each digit
 {1,1,1,1,1,1,0},                      //   the digit "0"
 {0,1,1,0,0,0,0},                      //   the digit "1"
 {1,1,0,1,1,0,1},                      //   the digit "2"
 {1,1,1,1,0,0,1},                      //   the digit "3"
 {0,1,1,0,0,1,1},                      //   the digit "4"
 {1,0,1,1,0,1,1},                      //   the digit "5"
 {1,0,1,1,1,1,1},                      //   the digit "6"
 {1,1,1,0,0,0,0},                      //   the digit "7"
 {1,1,1,1,1,1,1},                      //   the digit "8"
 {1,1,1,0,0,1,1},                      //   the digit "9"
 {0,0,0,0,0,0,0}                       // attempting to display the digit "10" turns all of the segments off
};

int OffMsgPointer = 0;                 // a variable indicating which letter of the word "OFF" is currently being displayed
int LCDoffMsg[13][7]={                 // an array indicating which segments are illuminated for each of the letters in the message " - OO FF FF" (which looks like the word "-OFF" scrolling through a one-character display
 {0,0,0,0,0,0,0},                      //   a blank space
 {0,0,0,0,0,0,1},                      //   the character  "-"
 {0,0,0,0,0,0,0},                      //   a blank space
 {0,0,1,1,1,0,1},                      //   the character  "O"
 {0,0,1,1,1,0,1},                      //   the character  "O"
 {0,0,0,0,0,0,0},                      //   a blank space
 {1,0,0,0,1,1,1},                      //   the character  "F"
 {1,0,0,0,1,1,1},                      //   the character  "F"
 {0,0,0,0,0,0,0},                      //   a blank space
 {1,0,0,0,1,1,1},                      //   the character  "F"
 {1,0,0,0,1,1,1},                      //   the character  "F"
 {0,0,0,0,0,0,0},                      //   a blank space
 {0,0,0,0,0,0,0}                       //   a blank space
};

int TestMsgPointer = 0;                // a variable indicating which letter of the word "TESTING" is currently being displayed
int LCDtestMsg[9][7]={                 // an array indicating which segments are illuminated for each of the letters in the message " TESTING "
 {0,0,0,0,0,0,0},                      //   a blank space
 {1,0,0,0,1,1,0},                      //   the character  "T"
 {1,0,0,1,1,1,1},                      //   the character  "E"
 {1,0,1,1,0,1,1},                      //   the character  "S"
 {1,0,0,0,1,1,0},                      //   the character  "T"
 {0,0,0,0,1,1,0},                      //   the character  "I"
 {1,1,1,0,1,1,0},                      //   the character  "N"
 {1,0,1,1,1,1,0},                      //   the character  "G"
 {0,0,0,0,0,0,0}                       //   a blank space
};

extern PGM_P *NApowerCodes[] PROGMEM;
extern PGM_P *EUpowerCodes[] PROGMEM;
extern uint8_t num_NAcodes, num_EUcodes;

/* This function is the 'workhorse' of transmitting IR codes.
 Given the on and off times, it turns on the PWM output on and off
 to generate one 'pair' from a long code. Each code has ~50 pairs! */
void xmitCodeElement(uint16_t ontime, uint16_t offtime, uint8_t PWM_code )
{
  TCNT2 = 0;
  if(PWM_code) {
    pinMode(IRLED, OUTPUT);
    // Fast PWM, setting top limit, divide by 8
    // Output to pin 3
    TCCR2A = _BV(COM2A0) | _BV(COM2B1) | _BV(WGM21) | _BV(WGM20);
    TCCR2B = _BV(WGM22) | _BV(CS21);
  } 
  else {
    // However some codes dont use PWM in which case we just turn the IR
    // LED on for the period of time.
    digitalWrite(IRLED, HIGH);
  }

  // Now we wait, allowing the PWM hardware to pulse out the carrier 
  // frequency for the specified 'on' time
  delay_ten_us(ontime);

  // Now we have to turn it off so disable the PWM output
  TCCR2A = 0;
  TCCR2B = 0;
  // And make sure that the IR LED is off too (since the PWM may have 
  // been stopped while the LED is on!)
  digitalWrite(IRLED, LOW);

  // Now we wait for the specified 'off' time
  delay_ten_us(offtime);
}

/* This is kind of a strange but very useful helper function
 Because we are using compression, we index to the timer table
 not with a full 8-bit byte (which is wasteful) but 2 or 3 bits.
 Once code_ptr is set up to point to the right part of memory,
 this function will let us read 'count' bits at a time which
 it does by reading a byte into 'bits_r' and then buffering it. */

uint8_t bitsleft_r = 0;
uint8_t bits_r=0;
PGM_P code_ptr;

// we cant read more than 8 bits at a time so dont try!
uint8_t read_bits(uint8_t count)
{
  uint8_t i;
  uint8_t tmp=0;

  // we need to read back count bytes
  for (i=0; i<count; i++) {
    // check if the 8-bit buffer we have has run out
    if (bitsleft_r == 0) {
      // in which case we read a new byte in
      bits_r = pgm_read_byte(code_ptr++);
      // and reset the buffer size (8 bites in a byte)
      bitsleft_r = 8;
    }
    // remove one bit
    bitsleft_r--;
    // and shift it off of the end of 'bits_r'
    tmp |= (((bits_r >> (bitsleft_r)) & 1) << (count-1-i));
  }
  // return the selected bits in the LSB part of tmp
  return tmp; 
}


/*
The C compiler creates code that will transfer all constants into RAM when 
 the microcontroller resets.  Since this firmware has a table (powerCodes) 
 that is too large to transfer into RAM, the C compiler needs to be told to 
 keep it in program memory space.  This is accomplished by the macro PROGMEM 
 (this is used in the definition for powerCodes).  Since the C compiler assumes 
 that constants are in RAM, rather than in program memory, when accessing
 powerCodes, we need to use the pgm_read_word() and pgm_read_byte macros, and 
 we need to use powerCodes as an address.  This is done with PGM_P, defined 
 below.  
 For example, when we start a new powerCode, we first point to it with the 
 following statement:
 PGM_P thecode_p = pgm_read_word(powerCodes+i);
 The next read from the powerCode is a byte that indicates the carrier 
 frequency, read as follows:
 const uint8_t freq = pgm_read_byte(code_ptr++);
 After that is a byte that tells us how many 'onTime/offTime' pairs we have:
 const uint8_t numpairs = pgm_read_byte(code_ptr++);
 The next byte tells us the compression method. Since we are going to use a
 timing table to keep track of how to pulse the LED, and the tables are 
 pretty short (usually only 4-8 entries), we can index into the table with only
 2 to 4 bits. Once we know the bit-packing-size we can decode the pairs
 const uint8_t bitcompression = pgm_read_byte(code_ptr++);
 Subsequent reads from the powerCode are n bits (same as the packing size) 
 that index into another table in ROM that actually stores the on/off times
 const PGM_P time_ptr = (PGM_P)pgm_read_word(code_ptr);
 */

uint16_t ontime, offtime;
uint8_t i,j, Loop;
uint8_t region = US;     // by default our code is US

void DisplayDigit(int DigitToShow) {                 // Displays the digit given by "DigitToShow"; calling for "10" blanks the display
  digitalWrite(SegA, LCDdigit[DigitToShow][0]);
  digitalWrite(SegB, LCDdigit[DigitToShow][1]);
  digitalWrite(SegC, LCDdigit[DigitToShow][2]);
  digitalWrite(SegD, LCDdigit[DigitToShow][3]);
  digitalWrite(SegE, LCDdigit[DigitToShow][4]);
  digitalWrite(SegF, LCDdigit[DigitToShow][5]);
  digitalWrite(SegG, LCDdigit[DigitToShow][6]);
  
}

void IncrementOffMsg() {                             // Displays the next character in the "- OFF" message
  digitalWrite(SegA, LCDoffMsg[OffMsgPointer][0]);
  digitalWrite(SegB, LCDoffMsg[OffMsgPointer][1]);
  digitalWrite(SegC, LCDoffMsg[OffMsgPointer][2]);
  digitalWrite(SegD, LCDoffMsg[OffMsgPointer][3]);
  digitalWrite(SegE, LCDoffMsg[OffMsgPointer][4]);
  digitalWrite(SegF, LCDoffMsg[OffMsgPointer][5]);
  digitalWrite(SegG, LCDoffMsg[OffMsgPointer][6]);
  OffMsgPointer ++;
  if (OffMsgPointer > 12) {OffMsgPointer = 0;}
  
}

void IncrementTestMsg() {                             // Displays the next character in the "TESTING" message
  digitalWrite(SegA, LCDtestMsg[TestMsgPointer][0]);
  digitalWrite(SegB, LCDtestMsg[TestMsgPointer][1]);
  digitalWrite(SegC, LCDtestMsg[TestMsgPointer][2]);
  digitalWrite(SegD, LCDtestMsg[TestMsgPointer][3]);
  digitalWrite(SegE, LCDtestMsg[TestMsgPointer][4]);
  digitalWrite(SegF, LCDtestMsg[TestMsgPointer][5]);
  digitalWrite(SegG, LCDtestMsg[TestMsgPointer][6]);
  TestMsgPointer ++;
  if (TestMsgPointer > 8) {TestMsgPointer = 0;}
  
}

void sendAllCodes() {                                 // Sends all of the "off" codes in the TV-B-Gone library (or up to the point where the test was stopped)
  // for every POWER code in our collection
  i = 0;
  do {
   PGM_P data_ptr; 

    // print out the code # we are about to transmit
    /* DEBUGP(putstring("\n\r\n\rCode #: "); 
    putnum_ud(i)); */

    // point to next POWER code, from the right database
    if (region == US) {
      data_ptr = (PGM_P)pgm_read_word(NApowerCodes+i);  
    } 
    else {
      data_ptr = (PGM_P)pgm_read_word(EUpowerCodes+i);  
    }

    // print out the address in ROM memory we're reading
    /* DEBUGP(putstring("\n\rAddr: "); 
    putnum_uh((uint16_t)data_ptr)); */

    // Read the carrier frequency from the first byte of code structure
    const uint8_t freq = pgm_read_byte(data_ptr++);
    // set OCR for Timer1 to output this POWER code's carrier frequency
    OCR2A = freq; 
    OCR2B = freq / 3; // 33% duty cycle

    // Print out the frequency of the carrier and the PWM settings
    /* DEBUGP(putstring("\n\rOCR1: "); 
    putnum_ud(freq);
    ); */
    /* DEBUGP(uint16_t x = (freq+1) * 2; 
    putstring("\n\rFreq: "); 
    putnum_ud(F_CPU/x);
    ); */

    // Get the number of pairs, the second byte from the code struct
    const uint8_t numpairs = pgm_read_byte(data_ptr++);
    /* DEBUGP(putstring("\n\rOn/off pairs: "); 
    putnum_ud(numpairs)); */

    // Get the number of bits we use to index into the timer table
    // This is the third byte of the structure
    const uint8_t bitcompression = pgm_read_byte(data_ptr++);
    /* DEBUGP(putstring("\n\rCompression: "); 
    putnum_ud(bitcompression);
    putstring("\n\r")); */

    // Get pointer (address in memory) to pulse-times table
    // The address is 16-bits (2 byte, 1 word)
    PGM_P time_ptr = (PGM_P)pgm_read_word(data_ptr);
    data_ptr+=2;
    code_ptr = (PGM_P)pgm_read_word(data_ptr);
    
    // Transmit all codeElements for this POWER code 
    // (a codeElement is an onTime and an offTime)
    // transmitting onTime means pulsing the IR emitters at the carrier 
    // frequency for the length of time specified in onTime
    // transmitting offTime means no output from the IR emitters for the 
    // length of time specified in offTime

#if 0

    // print out all of the pulse pairs
    for (uint8_t k=0; k<numpairs; k++) {
      uint8_t ti;
      ti = (read_bits(bitcompression)) * 4;
      // read the onTime and offTime from the program memory
      ontime = pgm_read_word(time_ptr+ti);
      offtime = pgm_read_word(time_ptr+ti+2);
      /* DEBUGP(putstring("\n\rti = "); 
      putnum_ud(ti>>2); 
      putstring("\tPair = "); 
      putnum_ud(ontime)); */
      /* DEBUGP(putstring("\t"); 
      putnum_ud(offtime)); */
    }
    continue;
#endif

    // For EACH pair in this code....
    cli();
    for (uint8_t k=0; k<numpairs; k++) {
      uint16_t ti;

      // Read the next 'n' bits as indicated by the compression variable
      // The multiply by 4 because there are 2 timing numbers per pair
      // and each timing number is one word long, so 4 bytes total!
      ti = (read_bits(bitcompression)) * 4;

      // read the onTime and offTime from the program memory
      ontime = pgm_read_word(time_ptr+ti);  // read word 1 - ontime
      offtime = pgm_read_word(time_ptr+ti+2);  // read word 2 - offtime
      // transmit this codeElement (ontime and offtime)
      xmitCodeElement(ontime, offtime, (freq!=0)); 
    } 
    sei();

    //Flush remaining bits, so that next code starts
    //with a fresh set of 8 bits.
    bitsleft_r=0;	

    // delay 250 milliseconds before transmitting next POWER code
    //delay_ten_us(25000);
    delay_ten_us(10000);

    // visible indication that a code has been output.
    if (Testing) IncrementTestMsg(); else IncrementOffMsg();
  
  i++;  
  } while ((i < NumberOfCodesToUse) && (digitalRead(Button) == false));
  
  if (i < NumberOfCodesToUse) {
    NumberOfCodesToUse = i;
    
  }
    
  // flash the visible LED on PB0  4 times to indicate that we're done
  delay_ten_us(65500); // wait maxtime 
  delay_ten_us(65500); // wait maxtime 
  DisplayDigit(10);
  quickflashLEDx(4);
  if (Testing) {
    DoneForTheNight = false;
    WillBeInaciveAt = millis() + InactivityTimeOut;
    SleepCyclesLeft = 675;  // 10-Minute Real Counter

  } else {
    DoneForTheNight = true;
    while (DoneForTheNight) {
      enterSleep();
          
    }
  }
}


ISR(WDT_vect)
{
  if(f_wdt == 0)
  {
    f_wdt=1;
  }
  else
  {
    Serial.println("WDT Overrun!!!");
  }
}


void enterSleep(void)
{
  f_wdt = 0;
  set_sleep_mode(SLEEP_MODE_PWR_SAVE);  // Establishes how deaply the Arduino will sleep.
  
  sleep_enable();  // Readies the Arduino for sleep.
  sleep_mode();    // Puts the Arduino to sleep.
  
  // Now the Arduino is asleap.  It will sleep until the 8-second watch dog timer issues an interupt.
  
  sleep_disable(); // The program picks up here when the watch dog timer wakes the Arduino - this is the first command execd upon exiting the sleep mode.
  power_all_enable();
  
}


void setup()   {
  // Setup the watch dog counter to wake the Arduino after 8 second of sleep.  Putting the chip to sleep while it is counting down and after the TV is off will greatly extend the battery life.
  MCUSR &= ~(1<<WDRF);             // Clears the reset flag.
  WDTCSR |= (1<<WDCE) | (1<<WDE);  // Sets WDCE to enable updates to the Arduino's internal settings (active for 4 clock cycles).
  WDTCSR = 1<<WDP0 | 1<<WDP3;      // Sets new watchdog timeout prescaler value to 8.0 seconds.
  WDTCSR |= _BV(WDIE);             //  Enable the watch dog interrupt (note no reset).
    
  pinMode(Button, INPUT);          // Use a pull-up button arrangment: a pushbutton attached from pin 2 to 5V, and an external 10K pull-down resistor is attached from pin 2 to GND
  pinMode(SegA, OUTPUT);     
  pinMode(SegB, OUTPUT);     
  pinMode(SegC, OUTPUT);     
  pinMode(SegD, OUTPUT);     
  pinMode(SegE, OUTPUT);     
  pinMode(SegF, OUTPUT);     
  pinMode(SegG, OUTPUT);
  pinMode(SegDP, OUTPUT);

  TCCR2A = 0;
  TCCR2B = 0;
  
  digitalWrite(LED, LOW);
  digitalWrite(IRLED, LOW);
  pinMode(LED, OUTPUT);
  pinMode(IRLED, OUTPUT);
  pinMode(REGIONSWITCH, INPUT);
  pinMode(TRIGGER, INPUT);
  digitalWrite(REGIONSWITCH, HIGH); //Pull-up
  digitalWrite(TRIGGER, HIGH);

  delay_ten_us(5000);            // Let everything settle for a bit

  // determine region
  if (digitalRead(REGIONSWITCH)) {
    region = US; // US
    NumberOfCodesToUse = num_NAcodes;   // We may have different number of codes in either database
    /* DEBUGP(putstring_nl("US")); */

  } 
  else {
    region = EU;
    NumberOfCodesToUse = num_EUcodes;
    /* DEBUGP(putstring_nl("EU")); */

  }

  // Tell the user what region we're in  - 3 is US 4 is EU
  quickflashLEDx(3+region);

  // Indicate how big our database is
  /* DEBUGP(putstring("\n\rNA Codesize: "); 
  putnum_ud(num_NAcodes);
  ); */
  /* DEBUGP(putstring("\n\rEU Codesize: "); 
  putnum_ud(num_EUcodes);
  ); */

  WillBeInaciveAt = millis() + InactivityTimeOut;
  
}


void loop() {
  if (digitalRead(Button) == true) {                        // If the button is being pushed ...
    WillBeInaciveAt = millis() + InactivityTimeOut;
    if (!AlreadyDown) {                                           // If the button wasn't down the last time it was checked (indicating that this is a new "pressing" of the button)
      FirstDownAt = millis();                                     //   ... make note of the moment the button was first physically pushed down, used to create a de-bounce clock
      AlreadyDown = true;                                         //   ... note that the pressing has been recorded and the debounce clock has already started
      
    } else if ((millis() - FirstDownAt) > PressAndHoldDelay) {    // if this is NOT a new "pressing" (so the button was held down at last check), and the "press-and-hold" delay has expired (so the button has been down quite a while)
      ButtonWasHeld = true;                                       //  ... indicate that the button was pressed and held
     
    }
    
  } else {                                                        // if the button is NOT pressed ...
    if ((AlreadyDown) && ((millis() - FirstDownAt) > PressOnlyDelay) && ((millis() - FirstDownAt) < PressAndHoldDelay)) {  // but the button was pressed last time it was checked (meaning it was just released), and the debounce period has expired (so its been down a few moments), but the press-and-hold period has not (so it was only pressed, not held) ...
      ButtonWasPressed = true;                                                                                             //   ... indicate that the button was pressed (but not held)

      }

    AlreadyDown = false;                                         // Indicates that the button is no longer being physically held down, so the next physical press will register as a new event
    
  }

  if (ButtonWasPressed) {           //  If the button was held down throughout the debounce interval but released before the press-and-hold interval...
    ButtonWasPressed = false;       //    ... reset the variable so the next pressing of the button will be counted as a new event
    SleepCyclesLeft -= 75;          //  Reduce the sleep timer by 10 minutes
    if (SleepCyclesLeft == 0) SleepCyclesLeft = 675;  // cycles the timer back to 9 minutes if the user pushes the button again when the timer is already set to 1 minute

  }
  
  if (ButtonWasHeld) {           //  If the button was physically held down throughout the entire press-and-hold interval (NOTE- this code will begin to execute while the button is still being held!)...
    ButtonWasHeld = false;       //    ... reset the variable so the next pressing-and-holding of the button will be counted as a new event
    CountingDown = false;
    SleepCyclesLeft = 0;
    Testing = true;
    TestMsgPointer = 0;
    DisplayDigit(10);
    delay (1500);
    sendAllCodes();
    AlreadyDown = false;                //  This tricks the software into thinking that the button was released, so that the same physical pressing of the button will not be counted again.
    
  }
  
  if (millis() >= WillBeInaciveAt) {    // If the 10-second programming period has lapsed...
    DisplayDigit(10);                   //   ... blank the display
    CountingDown = true;                //   ... and begin the countdown
    
  }
  
  if (CountingDown) {                   // If the Arduino just woke from an 8-second nap and sees that it the timer is counting down... 
    SleepCyclesLeft -= 1;               //   ... deincrement the timers 8-second counter
    if (SleepCyclesLeft == 0) {         //   If the timer has run out...
      CountingDown = false;             //      ... stop counting down
      Testing = false;                  //      ... record that this is not a test
      sendAllCodes();                   //      ... and send all codes
      
    } else {                            //  if the timer has not run out...
      enterSleep();                     //     ... take another 8-second nap

    }
    
  } else {                              // if the 10-second programing period has not elapsed
    DisplayDigit(SleepCyclesLeft/75);   //   ... display the currently selected number of 10-minute intervals
    
  }
}


/****************************** LED AND DELAY FUNCTIONS ********/
// This function delays the specified number of 10 microseconds
// it is 'hardcoded' and is calibrated by adjusting DELAY_CNT 
// in main.h Unless you are changing the crystal from 8mhz, dont
// mess with this.
void delay_ten_us(uint16_t us) {
  uint8_t timer;
  while (us != 0) {
    // for 8MHz we want to delay 80 cycles per 10 microseconds
    // this code is tweaked to give about that amount.
    for (timer=0; timer <= DELAY_CNT; timer++) {
      NOP;
      NOP;
    }
    NOP;
    us--;
  }
}


// This function quickly pulses the decimal point LED (connected to PIN 13)
// This will indicate to the user that a code is being transmitted
void quickflashLED( void ) {
  digitalWrite(LED, HIGH);
  delay_ten_us(3000);   // 30 millisec delay
  digitalWrite(LED, LOW);
}

// This function just flashes the visible LED a couple times, used to
// tell the user what region is selected
void quickflashLEDx( uint8_t x ) {
  quickflashLED();
  while(--x) {
    delay_ten_us(15000);     // 150 millisec delay between flahes
    quickflashLED();
  }
}

